@@ -13,6 +13,7 @@ module Fog
         :google_client,
         :google_client_options,
         :google_extra_global_projects,
+        :google_exclude_projects,
         :google_key_location,
         :google_key_string,
         :google_json_key_location,
@@ -170,6 +171,7 @@ module Fog
       request :set_forwarding_rule_target
       request :set_global_forwarding_rule_target
       request :set_server_disk_auto_delete
+      request :set_server_machine_type
       request :set_server_metadata
       request :set_server_scheduling
       request :set_server_tags
@@ -212,6 +214,8 @@ module Fog
 
       request :deprecate_image
 
+      request :reset_windows_password
+
       model_path "fog/compute/google/models"
       model :server
       collection :servers
